import cairo
import numpy as np


class Context:
    def __init__(self, height, width):
        self.height = height
        self.width = width
        self._surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
        self._ctx = cairo.Context(self._surface)

    @property
    def ctx(self):
        return self._ctx

    def export(self):
        buf = self._surface.get_data()
        return np.ndarray(shape=(self.width, self.height, 4), dtype=np.uint8, buffer=buf)



import librosa
import turtle
import numpy as np
from wsl import *
set_display_to_host()


def draw_note(note, t, octave_scale):
    octave = int(int(note[1]) * (1 + (octave_scale / 100)))
    note = note[0]

    if note == 'A':  # draw a circle
        t.circle(octave)
    elif note == 'B':  # draw a square
        for _ in range(4):
            t.fd(octave)
            t.rt(90)
    elif note == 'C':  # draw a triangle
        for _ in range(3):
            t.fd(octave)
            t.lt(120)
    elif note == 'D':  # draw a octogon
        for _ in range(8):
            t.fd(octave)
            t.lt(45)
    elif note == 'E':  # draw a hexagon
        for _ in range(6):
            t.fd(octave)
            t.lt(60)
    elif note == 'F':  # draw a star
        for _ in range(5):
            t.rt(108)
            t.fd(octave)
            t.lt(36)
            t.fd(octave)
    # sharp notes, don't want the sharp char to cause issues
    elif note[0] == 'A':  # rotate angle right by octave
        t.rt(octave)
    elif note[0] == 'B':  # rotate angle left by octave
        t.lt(octave)
    elif note[0] == 'C':  # do a 180
        t.lt(180)
    elif note[0] == 'D':  # pick up pen to octave places forward
        t.up()
        t.fd(octave)
        t.down()
    elif note[0] == 'E':  # pick up pen to octave places back
        t.up()
        t.lt(180)
        t.fd(octave)
        t.down()
    elif note[0] == 'F':  # change the pen color to random color
        colors = ['red', 'orange', 'yellow', 'green', 'blue', 'indigo', 'violet']
        t.pencolor(colors[octave % len(colors)])

def notes_to_canvas(song_path, speed_scale, octave_scale):
    y, sr = librosa.load(librosa.ex('trumpet'), sr=22050)
    S = np.abs(librosa.stft(y))
    bars = librosa.hz_to_note(S)

    turtle.screensize(canvwidth=400, canvheight=400)
    t = turtle.Turtle()
    t.speed(6 * (1 + (speed_scale / 100)))
    t.hideturtle()

    for bar in bars:
        for note in bar:
            if '-' in note:
                draw_note(note.split('-'), t, octave_scale)



import random

from PyQt5 import QtCore, QtGui, QtWidgets
from random import randrange
from PyQt5.QtCore import *
from PyQt5.QtGui import *


class QtCanvas(QtWidgets.QWidget):
    def __init__(self, parent):
        super(QtCanvas, self).__init__(parent)
        self._pixmap = QPixmap()
        self.resize(400, 400)
        self.x = 200
        self.y = 200
        self.lines = []
        self.arcs = []
        self.grads = []
        self.brush = QBrush(Qt.red, Qt.SolidPattern)

    @property
    def pixmap(self):
        return self._pixmap

    @pixmap.setter
    def pixmap(self, pixmap):
        self._pixmap = pixmap.copy()
        self.update()
        size = self.pixmap.size()
        if size.isValid():
            self.resize(size)
        else:
            self.resize(400, 400)

    def paintEvent(self,e):
        event = randrange(100)
        if event < 50:
            self.add_line()
        elif event < 90:
            self.add_arc()
        else:
            self.add_gradient()

        painter = QtGui.QPainter(self)
        painter.drawPixmap(self.rect(), self.pixmap)

        for x, y, width, height, grad, pen in self.grads:
            painter.setPen(pen)
            painter.setBrush(QBrush(grad))
            painter.drawRect(x, y, width, height)

        for x1, y1, x2, y2, pen in self.lines:
            painter.setPen(pen)
            painter.drawLine(x1, y1, x2, y2)

        for x, y, width, height, start_angle, span_angle, pen in self.arcs:
            painter.setPen(pen)
            painter.drawArc(x, y, width, height,start_angle, span_angle)

    def add_line(self):
        new_x = randrange(400)
        new_y = randrange(400)
        self.lines.append(tuple([self.x, self.y, new_x, new_y, self.get_pen()]))
        self.x = new_x
        self.y = new_y

    def add_arc(self):
        new_x = randrange(400)
        new_y = randrange(400)
        height = abs(new_y - self.y)
        width = abs(new_x - self.x)
        start_angle = random.randrange(-90, 90) * 16
        span_angle = random.randrange(-360, 360) * 16
        self.arcs.append(tuple([self.x, self.y, width, height, start_angle, span_angle, self.get_pen()]))
        self.x = new_x
        self.y = new_y

    def add_gradient(self):
        x = randrange(400)
        y = randrange(400)
        width = randrange(400)
        height = randrange(400)
        grad = QLinearGradient(randrange(400), randrange(400), randrange(400), randrange(400))
        self.grads.append(tuple([x, y, width, height, grad, self.get_pen()]))

    def get_pen(self):
        event = randrange(100)
        if event < 15:
            return QPen(Qt.red, 2, Qt.SolidLine)
        elif event < 30:
            return QPen(Qt.green, 2, Qt.SolidLine)
        elif event < 45:
            return QPen(Qt.blue, 2, Qt.SolidLine)
        elif event < 60:
            return QPen(Qt.yellow, 2, Qt.SolidLine)
        elif event < 75:
            return QPen(Qt.black, 2, Qt.SolidLine)
        else:
            return QPen(Qt.white, 2, Qt.SolidLine)

    def refresh(self):
        self.update()



from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import Qt
import wsl


class Window(QMainWindow):
    def __init__(self):
        super().__init__()

        title = "Glorious painting of your music"

        top = 400
        left = 400
        width = 800
        height = 600

        self.setWindowTitle(title)
        self.setGeometry(top, left, width, height)
        self.image = QImage(self.size(), QImage.Format_RGB32)
        self.image.fill(Qt.white)
        menu_bar = self.menuBar()
        file_button = menu_bar.addMenu("File")
        save_action = QAction("Save", self)
        save_action.setShortcut("Ctrl + S")
        file_button.addAction(save_action)
        save_action.triggered.connect(self.save)
        self.draw_something()

    # makes the blank canvas
    def paintEvent(self, event):
        canvas_creation = QPainter(self)
        canvas_creation.drawImage(self.rect(), self.image, self.image.rect())

    # does the drawing
    def draw_something(self):
        painter = QPainter(self.image)
        painter.setPen(QPen(Qt.black, 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
        painter.drawLine(100, 100, 300, 300)
        self.update()

    # save method
    def save(self):
        file_path, _ = QFileDialog.getSaveFileName(self, "Save Image", "", "PNG(*.png)")

        # if file path is blank return back
        if file_path == "":
            return

        # saves image
        self.image.save(file_path + ".png")


if __name__ == "__main__":
    wsl.set_display_to_host()
    app = QApplication([])
    window = Window()
    window.show()

    app.exec()



from PIL import Image
import cairo
import math
import numpy as np
from art_generator.canvas import Canvas
from art_generator.circle import Circle
from art_generator.context import Context


def do_stuff():
    WIDTH, HEIGHT = 256, 256

    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, WIDTH, HEIGHT)
    ctx = cairo.Context(surface)

    ctx.scale(WIDTH, HEIGHT)  # Normalizing the canvas

    pat = cairo.LinearGradient(0.0, 0.0, 0.0, 1.0)
    pat.add_color_stop_rgba(1, 0.7, 0, 0, 0.5)  # First stop, 50% opacity
    pat.add_color_stop_rgba(0, 0.9, 0.7, 0.2, 1)  # Last stop, 100% opacity

    ctx.rectangle(0, 0, 1, 1)  # Rectangle(x0, y0, x1, y1)
    ctx.set_source(pat)
    ctx.fill()

    ctx.translate(0.1, 0.1)  # Changing the current transformation matrix

    ctx.move_to(0, 0)
    # Arc(cx, cy, radius, start_angle, stop_angle)
    ctx.arc(0.2, 0.1, 0.1, -math.pi / 2, 0)
    ctx.line_to(0.5, 0.1)  # Line to (x,y)
    # Curve(x1, y1, x2, y2, x3, y3)
    ctx.curve_to(0.5, 0.2, 0.5, 0.4, 0.2, 0.8)
    ctx.close_path()

    ctx.set_source_rgb(0.3, 0.2, 0.5)  # Solid color
    ctx.set_line_width(0.02)
    ctx.stroke()

    # surface.write_to_png("example.png")  # Output to PNG
    buf = surface.get_data()
    return np.ndarray(shape=(WIDTH, HEIGHT, 4), dtype=np.uint8, buffer=buf)


def blank_image():
    canvas = Canvas(256, 256)
    return canvas.export()


def cairo_image():
    return Image.fromarray(do_stuff())


def layer():
    canvas = Canvas(256, 256)
    canvas.layer(do_stuff())
    return canvas.export()


def circle():
    context = Context(500, 500)
    Circle(context, 200, 200, 100).draw()
    return Image.fromarray(context.export())



from math import pi


class Circle:
    def __init__(self, context, x, y, radius):
        self._context = context
        self._position = (x, y)
        self._radius = radius

    @property
    def context(self):
        return self._context

    @property
    def position(self):
        return self._position

    @property
    def radius(self):
        return self.radius

    def draw(self):
        self._context.ctx.arc(self._position[0], self._position[1], self._radius, 0, pi*2)
        self._context.ctx.fill()
